/* PC Based Time and Options Settings For Geiger Shield  (Ver 2.0)                    (10/28/11) 
 * TO USE: Load sketch, open Serial Monitor - 9600 baud NO LINE ENDING - answer + Enter
 * Note, changing the Time Zone will not affect the time shown here. It's only used with the GPS 
 */

#include "WProgram.h"                   // this is needed to compile with Rel. 0013
#include <Wire.h> 
#include <EEPROM.h>                     // ATmega328's EEPROM used to store settings
#define RTC_ADDR 0x68
#define DDMMYY            1             // used with DATE_FMT below
#define YYMMDD            2             // used with DATE_FMT below
#define MMDDYY            3             // used with DATE_FMT below

#define DATE_FMT         MMDDYY         // pick date format from #defines above
#define AM_PM_FMT        1              // 0 if 24H / 1 if 12H format

// EEPROM Address for menu inputs - common for RTC and GPS 
#define LONG_PERIOD_ADDR  0
#define SHORT_PERIOD_ADDR 2
#define LOG_PERIOD_ADDR   4
#define USV_RATIO_ADDR    6
#define ALARM_SET_ADDR    8
#define ZONE_ADDR         50

char timeString[10];                    // time format determined by #defines above
char dateString[14];                    // time format determined by #defines above
byte seconds,minutes,hours,day_of_week,days,months,years,DST,PM;
byte setMin,setHour,setDay,setMonth,setYear,setDOW;
byte serInStr[6];                         // array that holds the serial input string


void setup() {
  Serial.begin(9600);
  Wire.begin(); 
  //Set_Square_Wave(0);                 // (if needed) 0=1Hz, 1=4KHz, 2=8KHz, 3=32KHz
  Get_Time(DATE_FMT);                   // read the formatted current date and time
  Serial.println("CURRENT DATE & TIME . . .");
  Serial.print(dateString);
  Serial.print(" ");
  Serial.print(timeString);
  if (AM_PM_FMT){                       // print AM/PM if defined
    if (PM) Serial.print("pm"); 
    else Serial.print("am");
  }
  Serial.println("");
  if (DST) Serial.println("DST FLAG SET");
  else Serial.println("DST FLAG IS NOT SET"); 

  Serial.println("");
  Serial.println("CURRENT SETTIONS . . .");
  Get_Settings(); // Display current settings from EEPROM
  Serial.println("");


  Serial.print("Set Date/Time? (Y/N) => ");
  Serial_To_Int(serInStr);                // use just the string not the Int
  if (serInStr[0] == 'y' || serInStr[0] == 'Y'){
    Serial.println(serInStr[0]);

    Serial.print("Day Of Week (1=Mon) => ");
    setDOW = Serial_To_Int(serInStr);     // for the rest, get just the Int
    Serial.println(setDOW,DEC);

    Serial.print("Month => ");
    setMonth = Serial_To_Int(serInStr);
    Serial.println(setMonth,DEC);

    Serial.print("Date => ");
    setDay = Serial_To_Int(serInStr);
    Serial.println(setDay,DEC);

    Serial.print("Year (YY) => ");
    setYear = Serial_To_Int(serInStr);
    Serial.println(setYear,DEC);

    Serial.print("Hour (0-23) => ");
    setHour = Serial_To_Int(serInStr);
    Serial.println(setHour,DEC);

    Serial.print("Minutes (& set) => ");
    setMin = Serial_To_Int(serInStr);
    Serial.println(setMin,DEC);

    Set_Time(setMin,setHour,setDay,setMonth,setYear,setDOW); // do the deed!
  } 

  Get_Time(DATE_FMT);              // read the date and time again
  DST_Flag();                      // Set flag in RTC NVRAM if DST
  Serial.println("");
  if (Read_NVRAM(0)== 0) Serial.println("DST FLAG IS NOT SET");
  else Serial.println("DST FLAG IS SET"); 
  Serial.println("");

  Serial.print("Change Settings ? (Y/N) => ");
  Serial_To_Int(serInStr);                // use just the string not the Int
  if (serInStr[0] == 'y' || serInStr[0] == 'Y'){
    Serial.println(serInStr[0]);
    Serial.println("");

    Serial.print("DISP PERIOD (sec) => ");
    writeParam(Serial_To_Int(serInStr), LONG_PERIOD_ADDR);
    Serial.println(readParam(LONG_PERIOD_ADDR),DEC);

    Serial.print("DOSE MODE (1=ON, else OFF) => ");
    writeParam(Serial_To_Int(serInStr), SHORT_PERIOD_ADDR);
    Serial.println(readParam(SHORT_PERIOD_ADDR),DEC);

    Serial.print("LOGGING PERIOD (min) => ");
    writeParam(Serial_To_Int(serInStr), LOG_PERIOD_ADDR);
    Serial.println(readParam(LOG_PERIOD_ADDR),DEC);

    Serial.print("CPM TO uSv RATIO => ");
    writeParam(Serial_To_Int(serInStr), USV_RATIO_ADDR);
    Serial.println(readParam(USV_RATIO_ADDR),DEC);

    Serial.print("ALARM THRESHOLD (CPM) => ");
    writeParam(Serial_To_Int(serInStr), ALARM_SET_ADDR);
    Serial.println(readParam(ALARM_SET_ADDR),DEC);

    Serial.print("TIME ZONE (For GPS, add 12 for + zones) => ");
    writeParam(Serial_To_Int(serInStr), ZONE_ADDR);
    Serial.println(readParam(ZONE_ADDR),DEC);

    Serial.println("");
    Serial.println("NEW SETTIONS . . .");
    Get_Settings(); // Display current settings from EEPROM
    Serial.println("");
  }
}

void loop(){
  Get_Time(DATE_FMT);                // just loop displaying time
  Serial.print(dateString);
  Serial.print(" ");
  Serial.print(timeString);
  if (AM_PM_FMT){                       // print AM/PM if defined
    if (PM) Serial.print("pm"); 
    else Serial.print("am");
  }
  Serial.println("");
  delay(5000);
}

int Serial_To_Int(byte *str){ 
  // ONLY FOR ARDUINO SERIAL MONITOR IN IDE WHERE ALL CHARS PASSED AT ONCE! 
  // Read from serial and passs back string & and also return as an integer
  // Returns -1 if non-numerc input but string keeps chars.
  int intValue = 0;
  byte i = 0;
  byte invalid = false;

  while(!Serial.available());             // wait for input

  while (Serial.available()){             // have input
    str[i] = Serial.read();               // build string
    if (str[i] < '0' || str[i] > '9') invalid = true; // test for non-digits
    intValue = intValue * 10 + (str[i] - '0');        // build integer
    i++;
    delay(10);                            // MUST have this!
  }
  str[i] = '\0';                          // indicate end of read string
  if (invalid == true){                   // handle bad input
    return -1;
  }
  else return intValue;                   // return integer
}

///////////////////////////// DS1307 RTC Functions ////////////////////////////////

void Get_Time(byte format){         // get the time and date from RTC lib and format it
  byte hour12;                      // local vars

  memset(timeString,0,sizeof(timeString));  // initialize the strings
  memset(dateString,0,sizeof(dateString));
  Wire.beginTransmission(0x68); 
  Wire.send(0); 
  Wire.endTransmission(); 
  Wire.requestFrom(0x68, 7);            // request 7 bytes from DS1307
  seconds = bcdToDec(Wire.receive());      
  minutes = bcdToDec(Wire.receive());      
  hours = bcdToDec(Wire.receive());        
  day_of_week = Wire.receive();    
  days = bcdToDec(Wire.receive());          
  months = bcdToDec(Wire.receive());      
  years = bcdToDec(Wire.receive());  

  // deal with AM/PM global and 12 hour clock no matter what format
  if (hours >= 12) PM = true;
  else PM = false;
  if (hours > 12)hour12 = hours - 12;
  else hour12 = hours;
  if (hours == 0) hour12 = 12;

  // make time string
  if (AM_PM_FMT)AppendToString (hour12,timeString);        // add 12 hour time to string
  else AppendToString (hours,timeString);                  // add 24 hour time to string
  strcat(timeString,":");
  if (minutes < 10) strcat(timeString,"0");
  AppendToString (minutes,timeString);                     // add MINUTES to string
  strcat(timeString,":");
  if (seconds < 10) strcat(timeString,"0");
  AppendToString (seconds,timeString);                     // add SECONDS to string

    // make date string
  if (format == DDMMYY)AppendToString (days,dateString);          // add DAY to string
  else if(format == MMDDYY)  AppendToString (months,dateString);  // add MONTH to string  
  else {                                                          // add YEAR to string
    if (years < 10) strcat(dateString,"0");
    AppendToString (years,dateString);
  }  
  strcat(dateString,"/");
  if (format == MMDDYY)AppendToString (days,dateString);          // add DAY to string
  else AppendToString (months,dateString);                        // add MONTH to string  
  strcat(dateString,"/");
  if (format == YYMMDD)AppendToString (days,dateString);          // add DAY to string
  else {
    if (years < 10) strcat(dateString,"0");
    AppendToString (years,dateString);                            // add YEAR to string
  }
  if (Read_NVRAM(0)== 0) DST = 0;
  else DST = 1; 
}


void Set_Time(byte Min,byte Hour,byte Day,byte Month,byte Year, byte DOW){
  Hour = ((Hour/10)*16)+(Hour % 10);         // convert each arg from decimal to BCD
  Min = ((Min/10)*16)+(Min % 10);
  Day = ((Day/10)*16)+(Day % 10);
  Month = ((Month/10)*16)+(Month % 10);
  Year = ((Year/10)*16)+(Year % 10);

  Wire.beginTransmission(0x68);              // slave address of DS1307 is 68H
  Wire.send(0);                              // begin register address 00H
  Wire.send(0x00);  // defult to 0 seconds
  Wire.send(Min);   // set minutes
  Wire.send(Hour);  // set hour (24 hour clock)
  Wire.send(DOW);   // set day of week (can use for "day 1", "day 2", etc.
  Wire.send(Day);   // set date
  Wire.send(Month); // set month
  Wire.send(Year);  // set year
  Wire.endTransmission(); 
}


// This function is not used in this sketch
void DST_Check(){ // Automatically changes the time based on DST

  // check if entering DST (2nd Sun in Mar.) . . .
  if (months == 3 && day_of_week == 7 && days > 7 && days <= 14 && hours >= 2) {
    if (!Read_NVRAM(0)){                  // if not currently in DST 
      Adj_Hour(1);                        // set the clock ahead 1 hour
      Write_NVRAM(0,1);                   // set the DST flag in EEPROM
    }
  }
  // check if leaving DST (1st Sun in Nov.) . . .
  if (months == 11 && day_of_week == 7 && days <= 7 && hours >= 2) {
    if (Read_NVRAM(0)){                  // if currently in DST 
      Adj_Hour(-1);                      // set the clock back 1 hour
      Write_NVRAM(0,0);                   // clear the DST flag in EEPROM
    }
  }
}

void DST_Flag(){  // Sets the DST Flag when setting date / time
  // Does not allow for changing time / date on the days where DST changes
  byte DST_Set = false;                 // set if DST is set in RTC EEPROM
  byte In_DST = false;                  // assume not in DST until proven otherwise
  byte DOWinDST;                        // in DST based on 2nd Sun in Mar. or 1st in Nov.
  char nextSun;                         // the date of the next Sunday

  if (Read_NVRAM(0)) DST_Set = true ; // see if DST set in RTC EEPROM

  // Prequalify for in DST in the widest range (any date between 3/8 and 11/7) 
  if ((months == 3 && days >= 8) || (months > 3 && months < 11) || (months == 11 && days <= 7) ){
    // DST start could be as late as 3/14 and end as early as 11/1
    DOWinDST = true;                    // assume it's in DST until proven otherwise
    nextSun = days + (7 - day_of_week); // find the date of the next Sunday
    if (nextSun == days) nextSun += 7;  // take care of today being Sunday
    if (months == 3 && (days <= nextSun)) DOWinDST = false;     // it's before the 2nd Sun in March
    if (months == 11 && (days >= nextSun -7)) DOWinDST = false; // it's after the 1st Sun in Nov.
    if (DOWinDST){                      // DOW is OK so it's in DST
      In_DST = true;                    // set flag so not in DST can be tested
      Write_NVRAM(0,1);                 // set the DST flag in EEPROM
    }
  }
  // Assumed not in DST, clear DST flag . . .
  if (In_DST == false) {
    Write_NVRAM(0,0);                   // clear the DST flag in EEPROM
  }
}


// This function is not used in this sketch
void Adj_Hour (int UpDown){
  Wire.beginTransmission(0x68);              // slave address of DS1307 is 68H
  Wire.send(0x02);                           // Hours register address is 2H
  Wire.send(hours + UpDown);                 // adjust Hours up or down
  Wire.endTransmission(); 
}


void AppendToString (byte bValue, char *pString){ // appends a byte to string passed
  char tempStr[6];
  memset(tempStr,'\0',sizeof(tempStr));
  itoa(bValue,tempStr,10);
  strcat(pString,tempStr);
}


byte bcdToDec(byte val) {               // Convert binary coded decimal to normal decimal numbers
  return ((val / 16 * 10) + (val % 16));
}

// This function is not used in this sketch
void Write_NVRAM(int write_slot, int data){// write to the NV RAM on the DS1307 chip                       
  // 56 one Byte slots available. Battery must be connected to retain data
  if (write_slot > 55) write_slot = 0;       // stop from looping back and writing clock data
  Wire.beginTransmission(0x68);              // slave address of DS1307 is 68H
  Wire.send(8+write_slot);                   // RAM is from 08H to 3FH - offset slot #
  Wire.send(data);                           // write data to that slot
  Wire.endTransmission(); 
}


int Read_NVRAM(int read_slot){ // raed from NV RAM on the DS1307 chip                       
  int rdata;                                 // declare the return
  Wire.beginTransmission(0x68);              // slave address of DS1307 is 68H
  Wire.send(8+read_slot);                    // set the pointer to the slot #
  Wire.endTransmission();                    // done setting pointer
  Wire.requestFrom(0x68, 1);                 // request 1 bytes from DS1307 at the pointer
  if (Wire.available()) rdata = Wire.receive();
  return rdata;
}


// This function is not required in this sketch
void Set_Square_Wave(int rate){  // set the square wave output on pin 7 of the DS1307 chip
  // NOTE: Open collector - an external pullup resistor must be connected to this pin.
  rate = rate + 144;                         // add 0x90 (dec 144) to rate
  Wire.beginTransmission(0x68);              // write the control register
  Wire.send(0x07);                           // register address 07H)
  Wire.send(rate);                           // 0=1Hz, 1=4KHz, 2=8KHz, 3=32KHz
  Wire.endTransmission();
} 

void Get_Settings(){
  long EEPROM_Read;

  EEPROM_Read = readParam(LONG_PERIOD_ADDR);
  Serial.print("LONG PERIOD (sec) = ");
  Serial.println(EEPROM_Read,DEC);

  EEPROM_Read = readParam(SHORT_PERIOD_ADDR);
  Serial.print("SHORT PERIOD (sec) = ");
  Serial.println(EEPROM_Read,DEC);

  EEPROM_Read = readParam(LOG_PERIOD_ADDR);
  Serial.print("LOGGING PERIOD (min) = ");
  Serial.println(EEPROM_Read,DEC);

  EEPROM_Read = readParam(USV_RATIO_ADDR);
  Serial.print("CPM TO uSv RATIO = ");
  Serial.println(EEPROM_Read,DEC);

  EEPROM_Read = readParam(ALARM_SET_ADDR);
  Serial.print("ALARM THRESHOLD (CPM) = ");
  Serial.println(EEPROM_Read,DEC);

  EEPROM_Read = readParam(ZONE_ADDR);
  Serial.print("TIME ZONE (GPS) = ");
  int GMT_Offset = EEPROM_Read;
  if (GMT_Offset >12) GMT_Offset = (GMT_Offset - 12);
  else GMT_Offset = GMT_Offset * -1;
  Serial.println(GMT_Offset,DEC);
}


void writeParam(unsigned int value, unsigned int addr){ // Write menu entries to EEPROM
  unsigned int a = value/256;
  unsigned int b = value % 256;
  EEPROM.write(addr,a);
  EEPROM.write(addr+1,b);
}


unsigned int readParam(unsigned int addr){ // Read previous menu entries from EEPROM
  unsigned int a=EEPROM.read(addr);
  unsigned int b=EEPROM.read(addr+1);
  return a*256+b; 
}





